@@ -82,6 +82,89 @@ a1a <- earth(etitanic[,-2], etitanic[,2],  # equivalent but using earth.default
              degree=2, trace=1,
              glm=list(family=binomial))
 printh(a1a)
+plotmo(a1a)
+
+a1b <- earth(etitanic[,-2,drop=FALSE], etitanic[,2,drop=FALSE],
+             degree=2, trace=1,
+             glm=list(family=binomial))
+printh(a1b)
+plotmo(a1b)
+
+# test modvars for the example in the man page earth.object.Rd
+
+aform <- earth(survived ~ age + pclass + sqrt(age) - sex, data=etitanic)
+cat("\nattr(aform$terms, \"factors\")\n")
+print(attr(aform$terms, "factors"))
+cat("\na$modvars\n")
+print(aform$modvars)
+cat("\n")
+
+axy.dat <- data.frame(age=etitanic$age, pclass=etitanic$pclass, sqrt_age=sqrt(etitanic$age))
+axy <- earth(axy.dat, etitanic$survived)
+cat("\nattr(axy$terms, \"factors\")\n")
+print(attr(axy$terms, "factors"))
+cat("\na$modvars\n")
+print(axy$modvars)
+cat("\n")
+
+# x and y dataframes but with missing column names
+xdf_nonames <- etitanic[,-2,drop=FALSE]
+cat("original colnames of xdf_nonames:", paste(colnames(xdf_nonames)), "\n")
+ydf_nonames <- etitanic[,2,drop=FALSE]
+colnames(xdf_nonames) <- NULL # weird for a dataframe, but earth still works
+colnames(ydf_nonames) <- NULL
+earth_df_nonames <- earth(xdf_nonames, ydf_nonames,
+             degree=2, trace=1,
+             glm=list(family=binomial))
+cat("earth_df_nonames:\n")
+print(summary(earth_df_nonames))
+cat("\nearth_df_nonames$modvars\n")
+print(earth_df_nonames$modvars)
+options(warn=2)
+expect.err(try(plotmo(earth_df_nonames)), "Cannot determine which variables to plot")
+plotmo(earth_df_nonames, all1=TRUE, SHOWCALL=TRUE)
+options(warn=1)
+plotmo(earth_df_nonames, trace=1, SHOWCALL=TRUE)
+
+# xmat in canonical form (double matrix) but with missing column names
+xmat_nonames <- etitanic[,"age",drop=FALSE]
+xmat_nonames$pclass <- as.numeric(etitanic[,"pclass"])
+xmat_nonames <- as.matrix(xmat_nonames)
+cat("original colnames of xmat_nonames:", paste(colnames(xmat_nonames)), "\n")
+ymat_nonames <- as.numeric(etitanic[,"survived"])
+ymat_nonames <- as.matrix(ymat_nonames)
+colnames(xmat_nonames) <- NULL
+colnames(ymat_nonames) <- NULL
+earth_mat_nonames <- earth(xmat_nonames, ymat_nonames, degree=2, trace=1)
+cat("earth_mat_nonames:\n")
+print(summary(earth_mat_nonames))
+options(warn=2)
+expect.err(try(plotmo(earth_mat_nonames)), "Cannot determine which variables to plot")
+options(warn=1)
+plotmo(earth_mat_nonames)
+
+# xmat in canonical form (double matrix) but with some missing column names
+xmat_partial <- etitanic[,"age",drop=FALSE]
+xmat_partial$pclass <- as.numeric(etitanic[,"pclass"])
+xmat_partial$sibsp <- as.numeric(etitanic[,"sibsp"])
+xmat_partial <- as.matrix(xmat_partial)
+cat("original colnames of xmat_partial:", paste(colnames(xmat_partial)), "\n")
+colnames(xmat_partial) <- c("", "x2", "") # some column names are missing (earth will create them)
+ymat_partial <- as.numeric(etitanic[,"survived"])
+ymat_partial <- as.matrix(ymat_partial)
+colnames(ymat_partial) <- "yy"
+earth_mat_partialnames <- earth(xmat_partial, ymat_partial, degree=2, trace=1)
+cat("earth_mat_partialnames:\n")
+print(summary(earth_mat_partialnames))
+options(warn=2)
+expect.err(try(plotmo(earth_mat_partialnames)), "Cannot determine which variables to plot")
+options(warn=1)
+plotmo(earth_mat_partialnames)
+
+# use a partial column name that will cause a duplicate within gen.colnames
+colnames(xmat_partial) <- c("", "xmat_partial1", "")
+expect.err(try(earth(xmat_partial, ymat_partial, degree=2, trace=1)),
+           "Duplicate colname in xmat_partial (colnames are \"xmat_partial1\", \"xmat_partial1\", \"xmat_partial3\")")
 
 a2 <- earth(pclass ~ ., data=etitanic, glm=list(family=binomial), trace=1)
 printh(a2)
@@ -207,11 +290,32 @@ cat(format(a, colon.char="*"))
 cat("--- mars.to.earth.Rd ----------------------\n")
 example(mars.to.earth)
 library(mda)
-a <- mars(trees[,-3], trees[,3])
-a <- mars.to.earth(a)
-print(summary(a, digits = 2))
-printh(summary(a, digits=2))
-printh(summary(a, digits=2, style="bf"))
+mars.mod <- mars(trees[,-3], trees[,3])
+cat("print.default(mars.mod):\n")
+print.default(mars.mod)
+mars.to.earth.mod <- mars.to.earth(mars.mod)
+cat("print.default(mars.to.earth.mod):\n")
+print.default(mars.to.earth.mod)
+printh(mars.to.earth.mod)
+printh(summary(mars.to.earth.mod))
+printh(summary(mars.to.earth.mod, style="bf"))
+stopifnot(length(mars.mod$coeff) == length(mars.to.earth.mod$coeff))
+stopifnot(max(mars.mod$coeff - mars.to.earth.mod$coeff) < 1e-10)
+earth.mod <- earth(trees[,-3], trees[,3])
+stopifnot(length(mars.mod$coeff) == length(earth.mod$coeff))
+# coeff differences can be big because forward passes are different
+stopifnot(max(mars.mod$coeff - earth.mod$coeff) < .3)
+
+par(mfrow=c(3,4), mar=c(4, 3.2, 3, 3), mgp=c(1.6, 0.6, 0), par(cex = 0.7))
+plot(mars.to.earth.mod, which=c(1,3), do.par=FALSE)
+plotmo(mars.to.earth.mod, do.par=FALSE)
+mars.to.earth.mod2 <- update(mars.to.earth.mod)
+plot(mars.to.earth.mod2, which=c(1,3), do.par=FALSE)
+plotmo(mars.to.earth.mod2, do.par=FALSE)
+plot(earth.mod, which=c(1,3), do.par=FALSE)
+plotmo(earth.mod, do.par=FALSE)
+par(org.par)
+
 cat("--- plot.earth.models.Rd ----------------------\n")
 if (PLOT)
     example(plot.earth.models)
@@ -341,7 +445,7 @@ printh(predict(a, newdata=data.frame(Girth=10,Height=80)))
 predict.a1d <- predict(a, newdata=data.frame(Girth=10,Height=80))
 check.almost.equal(predict.a1d[1,1], 17.6035895926138, msg="predict.a1d[1,1]")
 check.almost.equal(predict.a1d[1,2], 117.603589592614, msg="predict.a1d[1,2]")
-expect.err(try(predict(a, newdata=10)), "get.earth.x from model.matrix.earth from predict.earth:\n       could not convert vector x to matrix because length(x) 1\n       is not a multiple of the number 2 of predictors")
+expect.err(try(predict(a, newdata=10)), "Could not convert vector x to matrix because length(x) 1\n       is not a multiple of the number 2 of predictors")
 cat("2 predict(a)\n")
 printh(head(predict(a, trace=1)))
 cat("3a predict(a, matrix(c(10,12), nrow=1, ncol=2))\n")
@@ -364,6 +468,12 @@ printh(predict(a, xpredict, trace=1))
 expect.err(try(predict(a, interval="pin")), "no prediction intervals because the earth model was not built with varmod.method")
 expect.err(try(earth(cbind(Volume, Volume + 100) ~ ., data = trees, nfold=3, ncross=3, varmod.method="lm")), "variance models are not supported for multiple response models")
 
+options(warn=2)
+# TODO column naming for the following maybe needs work?
+#      nresponse="cbind(Volume, Volume + 100)2"  is confusing (2 should be in brackets?)
+expect.err(try(plot(a)), "Defaulting to nresponse=1, see above messages")
+options(warn=1)
+
 # repeat but with x,y (not formula) call to earth
 
 x1 <- cbind(trees$Girth, trees$Height)
@@ -410,11 +520,11 @@ x <- iris[1:3,]
 x[2,]$Petal.Width <- as.numeric(NA) # Petal.Width is unused in the earth model
 predict.with.message("formula interface and matrix with NA in unused variable", iris.earth, newdata=x)
 
-cat("--- earth.predict with NAs, with matrix interface ---\n")
+cat("--- earth.predict with NAs, with xy interface ---\n")
 
 iris.earth <- earth(iris[,1:3], iris[,4])
 x <- iris[1,]
-predict.with.message("default interface and vector", iris.earth, newdata=x)
+predict.with.message("default interface and vector", iris.earth, newdata=x) # tests the "Fix: April 2010" in get.earthx()
 x$Sepal.Width <- as.numeric(NA)
 predict.with.message("default interface and vector with NA", iris.earth, newdata=x)
 x <- iris[1,]
@@ -438,25 +548,33 @@ a$selected.terms[earth:::reorder.earth(a)]
 cat("--- tests with ozone data ----------------------\n")
 
 ozone.test <- function(itest, sModel, x, y, degree=2, nk=51,
-                    plotit=PLOT, trace=0, smooth.col="red")
+                    plotit=PLOT, trace=0, smooth.col="red", print.mars=FALSE)
 {
     fite <- earth(x, y, degree=degree, nk=nk, trace=trace)
     fitm <- mars(x, y, degree=degree, nk=nk)
+    fitme <- mars.to.earth(fitm)
 
     cat("itest",
         sprint("%-3d", itest),
         sprint("%-32s", sModel),
-        "degree", sprint("%-2d", degree), "nk", sprint("%-3g", nk),
-        "nTerms",  sprint("%-2d", sum(fite$selected.terms != 0)),
-        "of", sprint("%-3d", nrow(fite$dirs)),
-        "GRSq", sprint("%4.2g", fite$grsq),
-        "GRSq ratio", fite$grsq/mars.to.earth(fitm)$grsq,
+        "degree",   sprint("%-2d",  degree), "nk", sprint("%-3g", nk),
+        "nTerms",   sprint("%-2d",  sum(fite$selected.terms != 0)),
+        "of",       sprint("%-3d",  nrow(fite$dirs)),
+        "RSq",      sprint("%4.2g", fite$rsq),
+        "GRSq",     sprint("%4.2g", fite$grsq),
+        "mars RSq", sprint("%4.2g", fitme$rsq),
+        "ratio",    sprint("%.2f",  fite$rsq / fitme$rsq),
+        "GRSq",     sprint("%4.2g", fitme$grsq),
+        "ratio",    sprint("%.2f",  fite$grsq / fitme$grsq),
         "\n")
-    caption <- paste("itest ", itest, ": ", sModel, " degree=", degree, " nk=", nk, sep="")
+    if(print.mars) {
+        fitme1 <- update(fitme) # generate model selection data
+        printh(summary(fitme1))
+        cat("\n")
+    }
     printh(summary(fite))
-    printh(summary(fite, style="bf"))
     if(plotit) {
-        fitme <- mars.to.earth(fitm)
+        caption <- paste("itest ", itest, ": ", sModel, " degree=", degree, " nk=", nk, sep="")
         plotmo(fite, caption=paste("EARTH", caption), trace=-1)
         plotmo(fitme, caption=paste("MARS", caption), trace=-1)
         plot(fite, npoints=500, smooth.col=smooth.col, caption=paste("EARTH", caption), info=TRUE)
@@ -483,8 +601,7 @@ cat("--Expect warning from mda::mars: NAs introduced by coercion\n") # why do we
 x.global <- cbind(wind, exp(humidity))
 y <- doy
 # smooth.col is 0 else get loess errors
-# trace==2 so we print the "Fixed rank deficient bx by removing 2 terms, 7 terms remain" message
-# TODO why are we getting the rank deficient message?
+# trace==2 so we see "Fixed rank deficient bx by removing 2 terms, 7 terms remain"
 itest <- itest+1; ozone.test(itest, "doy ~ wind+exp(humidity)", x.global, y, degree=1, nk=21, smooth.col=0, trace=2)
 
 x.global <- cbind(vh,wind,humidity,temp,ibh,dpg,ibt,vis,doy)
@@ -493,7 +610,7 @@ itest <- itest+1; ozone.test(itest, "O3~.", x.global, y, degree=2, nk=21)
 
 x.global <- cbind(vh,wind,humidity,temp,ibh,dpg,ibt,vis,doy)
 y <- O3
-itest <- itest+1; ozone.test(itest, "O3~., nk=51", x.global, y, degree=2, nk=51)
+itest <- itest+1; ozone.test(itest, "O3~., nk=51", x.global, y, degree=2, nk=51, print.mars=TRUE)
 
 detach(ozone1)
 
@@ -584,6 +701,7 @@ test.plot.earth.args <- function()
          main="legend.pos=0")
 }
 test.plot.earth.args()
+par(org.par)
 
 cat("--- test minspan --------------------------------\n")
 
@@ -809,6 +927,7 @@ test.subset <- (1:nrow(ozone1))[-train.subset]
 # all the following models should be identical
 a <- earth(ozone1[,-1], ozone1[,1], subset=train.subset, nprune=7, degree=2)
 printh(a)
+plot(a)
 if (PLOT)
     plotmo(a, caption="test subset: earth(ozone1[,-1], ozone1[,1], subset=train.subset)", trace=-1)
 
@@ -851,7 +970,6 @@ x1 <- runif(10)
 x2 <- runif(10)
 y <- x1 + x2
 data=data.frame(x1=x1, x2=x2, y=y)
-OLD.PAR <- par(no.readonly=TRUE)
 par(mfrow = c(6, 4), mar = c(3, 3, 3, 1), mgp = c(1.5, 0.5, 0))
 
 expect.err(try(earth(y~., data=data, Auto.linpr=99)), "Auto.linpreds=99 but it should be FALSE, TRUE, 0, or 1")
@@ -909,7 +1027,7 @@ plotmo(a5, extend=.3, ylim=c(.2, 1.7),
 empty.plot()
 empty.plot()
 stopifnot(isTRUE(all.equal(as.vector(predict(a1)), as.vector(predict(a5)))))
-par(OLD.PAR)
+par(org.par)
 
 # more complicated example (with Auto.linpreds=TRUE, vh enters linearly in a degree2 term)
 data(ozone1)
@@ -921,7 +1039,6 @@ print(summary(mod.none2))
 stopifnot(all.equal(predict(mod.none1), predict(mod.none2)))
 
 # example figure in inst/doc
-OLD.PAR <- par(no.readonly=TRUE)
 par(mfrow=c(2,2), mar=c(4, 3.2, 3, 3), mgp=c(1.6, 0.6, 0), par(cex = 0.7))
 set.seed(2017)
 offset <- 98
@@ -949,7 +1066,7 @@ legend(x="topleft", legend=c("data", "earth model"),
        lty=c(0, 1), lwd=c(0, 2), pch=c(20, NA), col=c("red", 1))
 text(x=offset+4, y=offset-2.4, cex=.9, "Same data as previous graph")
 stopifnot(isTRUE(all.equal(predict(autolinTRUE), predict(autolinFALSE))))
-par(OLD.PAR)
+par(org.par)
 
 # test Auto.linpreds with data sent in by a user
 ndata <- matrix(data=c(
@@ -990,6 +1107,8 @@ colnames(ndata) <- c("x1", "x2", "x3", "x4", "x5", "y")
 ndata <- as.data.frame(ndata)
 
 cat("Auto.linpreds=TRUE pmethod=\"none\":\n")
+# trace==2 so we see "Fixed rank deficient bx by removing terms"
+# TODO why are we getting the rank deficient message?
 auto.linpreds.true.pmethod.none <- earth(y~., data=ndata, degree=2, nk=21, trace=2, pmethod="none")
 print(summary(auto.linpreds.true.pmethod.none, decomp="none"))
 cat("\nAuto.linpreds=FALSE pmethod=\"none\":\n")
@@ -1115,7 +1234,7 @@ a.formula.7update <- update(a.formula, wp=2, trace=1)
 a.formula.7  <- earth(Volume ~ ., wp=2, data = trees, keepxy=TRUE)
 check.models.equal(a.formula.7update, a.formula.7, msg="a7update a7", newdata=newdata.global)
 
-cat("\n----- update and keepxy, matrix interface--------------------------\n")
+cat("\n----- update and keepxy, xy interface--------------------------\n")
 
 Volume <- trees$Volume
 x <- cbind(trees$Height, trees$Volume)
@@ -1327,7 +1446,12 @@ cat("response weights: wp", wp, "earth gcv", e3$gcv,
     "mars gcv", m3$gcv, "mars gcv*length(wp)",
     m3$gcv * length(wp), "\n")
 
-e4 <- earth(cbind(O3, O3) ~ ., data=ozone1, wp=c(1, .01))
+expect.err(try(earth(cbind(O3, O3) ~ ., data=ozone1, wp=c(1, .01))),
+           "Duplicate colname in cbind(O3, O3) (colnames are \"O3\", \"O3\")")
+
+oz2 <- ozone1
+oz2$O3a <- ozone1$O3
+e4 <- earth(cbind(O3, O3a) ~ ., data=oz2, wp=c(1, .01))
 printh(e4) # both sub models should be the same
 printh(summary(e4))
 
@@ -1619,7 +1743,7 @@ cat("summary(a.xy1.trees, newdata=trees[1:5,])\n")
 a.xy1.trees.summary <- print(summary(a.xy1.trees, newdata=trees[1:5,]))
 stopifnot(a.xy1.trees.summary$newrsq == a.trees.summary$newrsq)
 
-cat("--- ../../tests/test.earth.R -------------------------\n")
+cat("--- /a/r/earth/tests/test.earth.R -------------------------\n")
 
 options(options.old)
 source("../../tests/test.earth.R")
@@ -1863,6 +1987,42 @@ names(xnew) <- NULL
 bx <- model.matrix(earth.girth.xy, xnew)
 check.model.matrix("earth.girth.xy x,y 15", xnew, bx, earth.girth.xy$bx[3,,drop=FALSE])
 
+cat("--- example in earth vignette \"How do I get p values for earth model coefficients?\" ---\n")
+
+earth.mod <- earth(Volume~., data=trees) # standard earth model
+bx <- earth.mod$bx[,-1]                  # earth model's basis mat (-1 to drop intercept)
+bx <- as.data.frame(bx)                  # lm requires a data frame
+bx$Volume <- trees$Volume                # add Volume to data
+lm.mod <- lm(Volume~., data=bx)          # standard linear regression on earth's basis mat
+summary(lm.mod)                          # prints p values
+
+remove(earth.mod, bx, lm.mod) # tidy up
+
+cat("--- examples in model.matrix.earth.Rd ---------------------------------------\n")
+
+# Example 1
+data(trees)
+earth.mod <- earth(Volume ~ ., data = trees) # standard earth model
+summary(earth.mod, decomp = "none")   # "none" to print terms in same order as lm.mod below
+
+bx <- model.matrix(earth.mod)         # equivalent to bx <- earth.mod$bx
+lm.mod <- lm(trees$Volume ~ bx[,-1])  # -1 to drop intercept
+summary(lm.mod)                       # yields same coeffs as above summary
+                                      # displayed t values are not meaningful
+
+# Example 2
+earth.mod <- earth(Volume~., data=trees) # standard earth model
+summary(earth.mod, decomp = "none") # "none" to print terms in same order as lm.mod below
+bx <- model.matrix(earth.mod)       # earth model's basis mat (equivalent to bx <- earth.mod$bx)
+bx <- bx[, -1, drop=FALSE]          # -1 to drop intercept
+bx <- as.data.frame(bx)             # lm requires a data frame
+bx$Volume <- trees$Volume           # add Volume to data
+lm.mod <- lm(Volume~., data=bx)     # standard linear regression on earth's basis mat
+summary(lm.mod)                     # yields same coeffs as above summary
+                                    # displayed t values are not meaningful
+
+remove(earth.mod, bx, lm.mod) # tidy up
+
 cat("--- compare backward, none, exhaustive, forward, seqrep ---------------------\n")
 data(ozone1)
 oz <- ozone1[1:50,]
@@ -1903,4 +2063,42 @@ earth.Species <- earth(Sepal.Length~Species, data=iris)
 predict.earth <- predict(earth.Species, newdata=data.frame(Species="setosa")) # used to fail
 stopifnot(identical(as.vector(predict.lm), as.vector(predict.earth)))
 
+# Check fix for bug reported by Max Kuhn (Oct 2020, fixed in earth 5.3.0):
+# Occasionally we used to put a 1 when we should have put a 2 into the dirs matrix.
+options.old <- options()
+options(width=1000)
+
+library(modeldata)
+data(ames)
+vars <- c("Sale_Price", "Gr_Liv_Area", "Alley", "Mas_Vnr_Type", "BsmtFin_Type_2", "Condition_2")
+ames2 <- ames[,vars,drop=FALSE]
+ames2$Sale_Price <- log10(ames2$Sale_Price)
+# change colnames to something easier to work with
+colnames(ames2) <- c("Sale_Price", "g", "a", "m", "b", "c")
+ames2 <- as.data.frame(ames2)
+ames2.mod <- earth(Sale_Price ~ ., data = ames2, degree = 2,
+                   trace=4, pmethod="none")
+cat("\nsummary(ames2.mod)\n")
+print(summary(ames2.mod))
+cat("\names2.mod$dirs\n")
+print(ames2.mod$dirs)
+plotmo(ames2.mod, SHOWCALL=TRUE)
+# check that there are no 1s in dirs, except for the "g" variable
+# all entries should be 0 or 2, because all vars are indicators (binary), so no knots
+stopifnot(all(ames2.mod$dirs[,-1,drop=FALSE] != 1)) # -1 drops "g" column
+stopifnot(ames2.mod$dirs["h(g-3390)*mStone",  "mStone"] == 2)
+
+# same as above but with Auto.linpreds=FALSE
+ames2.mod.Auto.linpreds.FALSE <- earth(Sale_Price ~ ., data = ames2, degree = 2,
+                                       pmethod="none", Auto.linpreds=FALSE)
+cat("\nsummary(ames2.mod.Auto.linpreds.FALSE)\n")
+print(summary(ames2.mod.Auto.linpreds.FALSE))
+cat("\nAuto.linpreds.FALSE$dirs\n")
+print(ames2.mod.Auto.linpreds.FALSE$dirs)
+# check that there are no 2s in dirs with Auto.linpreds=FALSE
+stopifnot(all(ames2.mod.Auto.linpreds.FALSE$dirs != 2))
+stopifnot(abs(ames2.mod$rsq - ames2.mod.Auto.linpreds.FALSE$rsq) < 1e-10)
+
+options(options.old) # no more width=1000
+
 source("test.epilog.R")
